(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: LOGGRAF.PAS                                                    *)
(*  Obsah: program pro kresleni grafu v linearnim a logaritmickem meritku  *)
(*  Autor: Mircosoft (http://mircosoft.mzf.cz)                             *)
(*  Posledni uprava: 16.4.2008                                             *)
(*  Pro kompilaci: VESA.TPU, RETEZCE.TPU, KLAVESY.TPU                      *)
(*  Pro spusteni: FONT8X16.FNT                                             *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
program Logaritmicko_linearni_graf;
uses vesa, {grafika}
     retezce, {pro prevody cisel na retezce}
     klavesy; {pro cteni stisknute klavesy - lze nahradit jednotkou CRT}

type TypHodnot=longint; {lze libovolne menit (i na realna cisla)}

{vstupni hodnoty (lze zadat jakekoli jine, i v jinem poctu):}
const PocetHodnot=17;
      hodnoty:array[1..pocethodnot] of typhodnot= {yove hodnoty pro vyneseni do grafu (zadavejte pouze kladna cisla!)}
         (1,10,50,100,500,1000,4000,9500,20000,40000,80000,160000,
          500000,1000000,50000000,1000000000,2000000000);
      deleniY=100000000; {Interval mezi dilky na ose y (uplatni se pouze pri linearnim meritku).
                          Lze libovolne menit, ale musi se rozsahem vejit do longintu.}

{volba typu grafu:}
      logaritmicky:boolean=false; {timhle se prepina, jestli ma byt yova osa logaritmicka nebo linearni}

{dalsi nastaveni (kosmeticke detaily):}
      rozliseni=_640x480; {lze libovolne menit, meritko grafu se prizpusobi}
      font='font8x16.fnt'; {lze pouzit i jiny}
      BarvaOs=15;
      BarvaPozadi=0;
      LevyOkraj=100;
      PravyOkraj=10;   {sirka mezer mezi okraji oblasti grafu}
      HorniOkraj=10;   {a okraji obrazovky                   }
      DolniOkraj=40;
      BarvyProuzku:array[0..13] of byte=(1,12,2,14,3,13,4,5,15,6,7,11,9,10);

var f:_font; {aby bylo cim psat}
    x0,y0:integer; {poloha pocatku souradnic na obrazovce (vypocita se podle rozliseni)}
    sirka,vyska:word; {rozmery oblasti grafu (vypocitaji se podle rozliseni)}
    delenix:word; {sirka dilku na vodorovne ose (vypocitaji se podle poctu hodnot)}
    meritkoy:real; {konstanta pro prepocty hodnot na meritko obrazovky}
    NejvetsiHodnota:typhodnot;
    PrepocitaneHodnoty:array[1..pocethodnot] of word; {uz v pixelech, staci vykreslit}
    w:word; {pomocne pocitadlo pro cykly}
    y:word;           {\                                        }
    MaxCifer:byte;    { \ pomocne promenne pro popisovani osy y }
    cislo:string[10]; { /                                       }
    pom:typhodnot;    {/                                        }
    znak:char; {pro cteni stisknute klavesy}

function log(x:typhodnot):real; {funkce pro desitkovy logaritmus}
Begin
if x<=0 then log:=0 {logaritmus nekladneho cisla neni definovan}
        else log:=ln(x)/ln(10);
End;{log}

BEGIN
{spusteni grafiky a inicializace pisma:}
_frekvence:=85; {lze vyradit}
_setmode(rozliseni);
_nactifont2(f,font);
_nastavfont(f,true);

{vypocty rozmeru oblasti grafu a polohy pocatku souradnic podle rozliseni
obrazovky a nastaveni okraju:}
sirka:=_maxx+1-levyokraj-pravyokraj;
vyska:=_maxy+1-horniokraj-dolniokraj;
delenix:=sirka div pocethodnot;
x0:=levyokraj+1;
y0:=_maxy-dolniokraj;

{nalezeni nejvetsi ze zadanych hodnot:}
nejvetsihodnota:=hodnoty[1];
for w:=2 to pocethodnot do if hodnoty[w]>nejvetsihodnota then nejvetsihodnota:=hodnoty[w];

 repeat

 {prepocet hodnot do prislusneho meritka, aby sly zobrazit:}
 if logaritmicky
   then begin
        meritkoy:=vyska/log(nejvetsihodnota);
        for w:=1 to pocethodnot do
         prepocitanehodnoty[w]:=round(log(hodnoty[w])*meritkoy);
          {hodnoty se ukladaji jako exponenty, tj. 10 na kolikatou}
        end
   else begin
        meritkoy:=vyska/nejvetsihodnota;
        for w:=1 to pocethodnot do
         prepocitanehodnoty[w]:=round(hodnoty[w]*meritkoy); {hodnoty se ukladaji primo}
        end;

 {kresleni grafu:}
 _fill(barvapozadi);
 __rectangle(x0-1,y0-vyska,x0+sirka,y0+1,barvaos);{okraje}
 {popisky osy y:}
 maxcifer:=length(nastr(round(nejvetsihodnota)));
 if logaritmicky
   then begin {logaritmicke meritko}
        w:=0; {w bude exponent, zacneme od 0 (10 na nultou = 1)}
        while w<=log(nejvetsihodnota) do
          begin
          y:=y0-round(meritkoy*w); {poloha rysky}
          __hline(x0-4,x0-1,y,barvaos); {ryska}
          cislo:=zarovnej('1'+kolikceho(w,'0'),maxcifer,_doprava,' '); {slozeni cisla
                 (10 na xtou znamena jednicka a za ni x nul) a jeho zarovnani doprava}
          _print(x0-_sirkatextu2(maxcifer)-10, {nekde dostatecne vlevo}
                 y-_vyskatextu2 shr 1, {o pul vysky pisma nad ryskou}
                 barvaos,cislo);
          inc(w); {jdeme na dalsi exponent}
          end;
        end
   else begin {linearni meritko}
        pom:=0; {pom bude cislo primo v meritku hodnot}
        while pom<=nejvetsihodnota do
          begin
          y:=y0-round(meritkoy*pom); {poloha rysky}
          __hline(x0-4,x0-1,y,barvaos); {ryska}
          cislo:=nastr2(round(pom),maxcifer); {prevod cisla na retezec a jeho zarovnani doprava}
          _print(x0-_sirkatextu2(maxcifer)-10,y-_vyskatextu2 shr 1,barvaos,cislo);
          pom:=pom+deleniy; {jdeme o kus vys}
          end;
        end;
 {vyneseni hodnot:}
 for w:=1 to pocethodnot do
  __bar(x0+(w-1)*delenix,
        y0-prepocitanehodnoty[w],
        x0+w*delenix-1,y0,
        barvyprouzku[w mod 14]); {barva se vybira z pripraveneho pole
            (pri primem dosazeni w za barvu by byly nektere prouzky cerne a nebyly by videt)}

 {interakce s uzivatelem:}
 if logaritmicky then _print(levyokraj,_maxy-4-2*_vyskatextu2,15,
                             'Osa y je v logaritmickm mtku - %%b012P%%Bepnout na linern')
                 else _print(levyokraj,_maxy-4-2*_vyskatextu2,15,
                             'Osa y je v linernm mtku - %%b012P%%Bepnout na logaritmick');
 _print(levyokraj,_maxy-1-_vyskatextu2,barvaos,'Konec stisknutm klvesy %%b012Esc%%Bape.');
 znak:=readkey;
 if upcase(znak)='P' then logaritmicky:=not logaritmicky;

 until znak=#27;

{zruseni pisma a ukonceni grafiky:}
_zrusfont(f);
_setmode(0);
END.